//===================================================== IBJ37838.c
//
// Boundary layer profile functions of VDI 3783 Part 8
// ===================================================
//
// Copyright (C) Janicke Consulting, Dunum, Germany, 2002
// email: info@janicke.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// last change:
// 2002-07-12 uj  1.0.1 Extracted from IBJprf 1.3.5
//
//==================================================================

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>

#define  PRF_FU    2.5
#define  PRF_FV    2.0
#define  PRF_FW    1.3
#define  PRF_FFU   0.17
#define  PRF_FFV   0.17
#define  PRF_FFW   0.13
#define  PRF_PS    4.7
#define  PRF_PN    1.0
#define  PRF_PL    15.0
#define  PRF_KAPPA 0.4
#define  PRF_ALPHA 0.3
#define  PRF_FCOR  1.e-4
#define  PRF_TMPGRADIENT -0.0098
#define  PRF_GRAV  9.81
#define  VDI_EX    1.0
#define  VDI_C0    5.7

extern char *PrfVersion;

float PrfGetU(float z, float lm, float hm, float ust, float z0, float interval);
float PrfGetSigU(float z, float lm, float hm, float ust, float z0, float interval);
float PrfGetSigV(float z, float lm, float hm, float ust, float z0, float interval);
float PrfGetSigW(float z, float lm, float hm, float ust, float z0, float interval);
float PrfGetKU(float z, float lm, float hm, float ust, float z0, float interval);
float PrfGetKV(float z, float lm, float hm, float ust, float z0, float interval);
float PrfGetKW(float z, float lm, float hm, float ust, float z0, float interval);
float PrfGetRa(float h, float lm, float hm, float z0);
float PrfGetHM(float lm, float ust);
void  PrfSetKappa(float kappa);
void  PrfSetD0(float d0);
void  PrfSetHa(float ha);
void  PrfSetRa(float ra);
void  PrfSetBeta(float beta);
void  PrfSetVersion(int version);
int   PrfGetVersion();
char *PrfGetPrgVersion();

#define VDI3783_8  2

extern char *PrfVersion = "1.0.1";

float Ha, Ua, Z0, Ra, Ust, Lm, Hm, Zz[101], Interval, D0=0., Kappa=PRF_KAPPA;
float Beta = 0.01;
int   Nz = 100;
char  Name[512];
int   Version = VDI3783_8;
int   ImplVersions[] = { VDI3783_8 };
int   NVersions = 1;
int   StdVersion = VDI3783_8;
int   DEBUG = 0;

//----------------------------------------------------------------- PrfSetKappa
void PrfSetKappa(float kappa) { Kappa = kappa; }

//-------------------------------------------------------------------- PrfSetD0
void PrfSetD0(float d0) { D0 = d0; }

//-------------------------------------------------------------------- PrfSetHa
void PrfSetHa(float ha) { Ha = ha; }

//-------------------------------------------------------------------- PrfSetRa
void PrfSetRa(float ra) { Ra = ra; }

//------------------------------------------------------------------ PrfSetBeta
void PrfSetBeta(float beta) { Beta = beta; }

//------------------------------------------------------------ PrfGetPrgVersion
char *PrfGetPrgVersion() { return PrfVersion; }


//--------------------------------------------------------------- PrfSetVersion
void PrfSetVersion(int version) {
  int i;
  Version = version;
  for (i=0; i<NVersions; i++) if (ImplVersions[i] == Version) break;
  if (i >= NVersions) Version = StdVersion;
  if (DEBUG) printf("Version set to %d\n", Version);
}

//--------------------------------------------------------------- PrfGetVersion
int PrfGetVersion() {
  return Version;
}

//--------------------------------------------------------------------- PrfGetU
float PrfGetU(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, zeta, zeta0, psi, x1, x2, x3, x, x0, u, hh;
  hh = D0+6*z0;
  z = (h < hh) ? hh-D0 : h-D0;
  z -= z0;
  if (z < 0) z = 0;
  zeta = (z+z0)/lm;
  zeta0 = z0/lm;
  if (lm > 0.) {
    x1 = -5*zeta0;
    x2 = 7*log(2.) - log(zeta0) - 5*zeta0 - 4;
    x3 = 8*log(20) - 11.165 - log(2.) - log(zeta0) - 5*zeta0;
    if (zeta < 0.5)
      psi = log(zeta/zeta0) + 5*zeta + x1;
    else if (zeta <= 10.)
      psi = 8*log(zeta) + 4.25/zeta - 0.5/zeta/zeta + x2;
    else
      psi = 0.7585*zeta + x3;
    u = ust/Kappa*psi;
  }
  else {
    x = pow(1 - 15*zeta, 0.25);
    x0 = pow(1 - 15*zeta0, 0.25);
    x1 = (1+x)*(1+x)/(1+x0)/(1+x0) * (1+x*x)/(1+x0*x0);
    psi = log(x1) - 2*(atan(x) - atan(x0));
    u = ust/Kappa*(log(zeta/zeta0) - psi);
  }
  if (h < hh) u *= h/hh;
  if (u < 0) u = 0.;
  return u;
}

//------------------------------------------------------------------ PrfGetSigU
float PrfGetSigU(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, sig, wst;
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  if (lm > 0) {
    sig = 2.4*ust*exp(-VDI_EX*z/hm);
  }
  else {
    wst = ust*pow(-hm/Kappa/lm, 1./3.);
    sig = pow(2.4*ust, 3.) + pow(0.59*wst, 3.);
    sig = pow(sig, 0.333333) * exp(-VDI_EX*z/hm);
  }
  if (sig < 0.01*ust) sig = 0.01*ust;
  return sig;
}

//------------------------------------------------------------------ PrfGetSigV
float PrfGetSigV(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, sig, wst;
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  if (lm > 0.) {
    sig = 1.8*ust*exp(-VDI_EX*z/hm);
  }
  else {
    wst = ust*pow(-hm/Kappa/lm, 1./3.);
    sig = pow(1.8*ust, 3.) + pow(0.59*wst, 3.);
    sig = pow(sig, 0.333333) * exp(-VDI_EX*z/hm);
  }
  if (sig < 0.01*ust) sig = 0.01*ust;
  return sig;
}

//------------------------------------------------------------------ PrfGetSigW
float PrfGetSigW(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, sig, wst, a;
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  a = z/hm;
  if (lm > 0.) {
    sig = 1.3*ust*exp(-VDI_EX*a);
  }
  else {
    wst = ust*pow(-hm/Kappa/lm, 0.333333);
    sig = pow(1.3*ust*exp(-VDI_EX*a), 3.);
    if (1-0.8*a > 0.)
      sig += pow(1.3*pow(a,0.333333)*(1-0.8*a)*wst, 3.);
    sig = pow(sig, 0.333333);
  }
  if (sig < 0.01*ust) sig = 0.01*ust;
  return sig;
}

//-------------------------------------------------------------------- PrfGetKW
float PrfGetE(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, f1, f2, a, rwsus3;
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  if (hm <= 0)  hm = 800;
  a = z/hm;
  f1 = 2.5*Kappa*a;
  if (a < 1)  f1 += (1-a)*(1-a);
  if (lm < 0) {
    rwsus3 = -hm/(Kappa*lm);
    f2 = 1.5 - 1.3*pow(a, 1./3.);
    if (f2 > 0)  f1 += rwsus3*a*Kappa*f2;
    if (f1 < 1.) f1 = 1.;  
  }
  else { 
    f1 = 1. + 4*z/lm;
  }
  return (f1*ust*ust*ust/(Kappa*z));
}

//-------------------------------------------------------------------- PrfGetKW
float PrfGetKW(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, k, sig, kmin, e, a;
  sig = PrfGetSigW(h, lm, hm, ust, z0, interval);
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  a = z/hm;
  e = PrfGetE(h, lm, hm, ust, z0, interval);
  k = 2*sig*sig*sig*sig/VDI_C0/e;
  kmin = Beta*sig*sig*z0/ust;
  if (k < kmin) k = kmin;
  return k;
}

//-------------------------------------------------------------------- PrfGetKU
float PrfGetKU(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, sig, k, kmin, e;
  sig = PrfGetSigU(h, lm, hm, ust, z0, interval);
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  e = PrfGetE(h, lm, hm, ust, z0, interval);
  k = 2*sig*sig*sig*sig/VDI_C0/e;
  kmin = Beta*sig*sig*z0/ust;
  if (k < kmin) k = kmin;
  return k;
}

//-------------------------------------------------------------------- PrfGetKV
float PrfGetKV(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, sig, k, kmin, e;
  sig = PrfGetSigV(h, lm, hm, ust, z0, interval);
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  e = PrfGetE(h, lm, hm, ust, z0, interval);
  k = 2*sig*sig*sig*sig/VDI_C0/e;
  kmin = Beta*sig*sig*z0/ust;
  if (k < kmin) k = kmin;
  return k;
}

//-------------------------------------------------------------------- PrfGetHM
float PrfGetHM(float lm, float ust) {
  float hm;
  hm = 1100.;
  if (lm > 0.) {
    hm = 0.7*sqrt(ust*lm/1.11e-4);
    if (hm > 800.) hm =  800.;
    if (hm < 250.)  hm = 250.;
  }
  return hm;
}

//------------------------------------------------------------------- PrfGetRa
float PrfGetRa(float h, float lm, float hm, float z0) {
  return Ra;
}  

#ifdef MAIN /*###############################################################*/
#define  ISOPTION(a)   ((a) == '-')

//-------------------------------------------------------------------- profiles
void profiles() {
  int i;
  float vx[40], su[40], sv[40], sw[40], ku[40], kv[40], kw[40], ra[40], z;
  FILE *fp;

  if (Nz <= 0) {
    Nz = 30;
    Zz[0] = 0.;
    i = 1;
    while (i++ < 20) Zz[i] = Zz[i-1] + 10.;
    while (i++ < 30) Zz[i] = Zz[i-1] + 30.;
    while (i++ < 40) Zz[i] = Zz[i-1] + 50.;
  }
  if (Ust == 0.) {
    if (Ha <= 0.) Ha = 10.;
    if (Ua <= 0.) Ua = 4.;
  }
  if (Ra <= 0.) Ra = 270.;
  if (Z0 <= 0.) Z0 = 1.0;
  if (Lm == 0.) Lm = 99999.;
  if (Ust <= 0.) Ust = Ua/PrfGetU(Ha, Lm, Hm, 1., Z0, Interval);

  if (Hm <= 0.) Hm = PrfGetHM(Lm, Ust);

  for (i=0; i<=Nz; i++) {
    z = Zz[i];
    vx[i] = PrfGetU(z, Lm, Hm, Ust, Z0, Interval);
    su[i] = PrfGetSigU(z, Lm, Hm, Ust, Z0, Interval);
    sv[i] = PrfGetSigV(z, Lm, Hm, Ust, Z0, Interval);
    sw[i] = PrfGetSigW(z, Lm, Hm, Ust, Z0, Interval);
    ku[i] = PrfGetKU(z, Lm, Hm, Ust, Z0, Interval);
    kv[i] = PrfGetKV(z, Lm, Hm, Ust, Z0, Interval);
    kw[i] = PrfGetKW(z, Lm, Hm, Ust, Z0, Interval);
    ra[i] = PrfGetRa(z, Lm, Hm, Z0);
  }

  remove(Name);
  fp = (strlen(Name) > 1) ? fopen(Name, "w") : stdout;
  fprintf(fp, "   ' IBJ37838 %s\n", PrfVersion);
  fprintf(fp, "   ' us=%8.4f lm=%8.2f hm=%8.2f\n", Ust, Lm, Hm);
  fprintf(fp, "   ' nz=%8.0d ha=%8.2f ua=%8.3f\n", Nz, Ha, Ua);
  fprintf(fp, "   ' z0=%8.4f d0=%8.2f ra=%8.3f\n", Z0, D0, Ra);
  fprintf(fp, "   ' kp=%8.2f Beta=%6.3f\n", Kappa, Beta);
  fprintf(fp, "zz  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", Zz[i]); fprintf(fp, "\n");
  fprintf(fp, "vx  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", vx[i]); fprintf(fp, "\n");
  fprintf(fp, "su  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", su[i]); fprintf(fp, "\n");
  fprintf(fp, "sv  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", sv[i]); fprintf(fp, "\n");
  fprintf(fp, "sw  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", sw[i]); fprintf(fp, "\n");
  fprintf(fp, "ku  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", ku[i]); fprintf(fp, "\n");
  fprintf(fp, "kv  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", kv[i]); fprintf(fp, "\n");
  fprintf(fp, "kw  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", kw[i]); fprintf(fp, "\n\n");
}

//------------------------------------------------------------------------ Help
void Help() {
  printf("Calculation of 1-dim. meteorological profiles according to VDI 3783 Part 8\n");
  printf("usage: IBJ37838 [options]\n");
  printf("options:\n");
  printf("  -d<number>   Displacement height (m, default: 0)\n");
  printf("  -k<number>   Kappa (default: 0.4)\n");
  printf("  -l<number>   Monin-Obukhov length (m, default: 99999)\n");
  printf("  -m<number>   Height of the mixing layer (m)\n");
  printf("  -h<number>   Anemometer height (m, default: 10)\n");
  printf("  -u<number>   Wind velocity at anemometer height (m/s, default: 4)\n");
  printf("  -s<number>   Shear velocity (m/s)\n");
  printf("  -z<number>   Roughness length (m, default: 1)\n");
  printf("  -r<number>   Wind direction at anemometer height (deg, default: 270)\n");
  printf("  -v<z1>,<z2>,...  Vertical grid (m)\n");
  printf("  -f<file>     Output file name\n");
  printf("\n");
  printf("example (guideline VDI 3783 Part 8, Fig. 4, unstable):\n");
  printf("  IBJ37738 -v200,500,900 -m1100 -s0.453 -l-190 -z1.5 -u2\n");
  exit(0);
}

//------------------------------------------------------------------------ main
void main( int argc, char *argv[] ) {
  int i, j;
  float z;
  char *pc;
  printf("IBJ37838, version %s of %s, %s\n", PrfVersion, __DATE__, __TIME__);
  printf("1-dim. boundary layer profiles according to VDI 3783 Part 8\n");
  printf("Copyright (C) Janicke Consulting, Dunum, Germany, 2002\n");
  printf("IBJ37838 comes with ABSOLUTELY NO WARRANTY\n");
  printf("IBJ37838 is free software under the GNU PUBLIC LICENCE\n\n");
  Lm = 0.;
  Hm = 0.;
  Ua = 0.;
  Ha = 0.;
  Ust = 0.;
  Nz = 0.;
  Z0 = 0.;
  D0 = 0.;
  Ra = 0.;
  Kappa = PRF_KAPPA;
  Interval = 3600.;
  sprintf(Name, "");
  if (argc < 2) Help();
  for (i=1; i<argc; i++) {
    pc = argv[i];
    if (ISOPTION(*pc)) {
      switch (pc[1]) {
        case 'l': j=sscanf(pc+2, "%f", &Lm);
                  if (j != 1) Help();
                  break;
        case 'm': j=sscanf(pc+2, "%f", &Hm);
                  if (j != 1) Help();
                  break;
        case 'u': j=sscanf(pc+2, "%f", &Ua);
                  if (j != 1) Help();
                  break;
        case 'h': j=sscanf(pc+2, "%f", &Ha);
                  if (j != 1) Help();
                  break;
        case 'd': j=sscanf(pc+2, "%f", &D0);
                  if (j != 1) Help();
                  break;
        case 'k': j=sscanf(pc+2, "%f", &Kappa);
                  if (j != 1) Help();
                  break;
        case 's': j=sscanf(pc+2, "%f", &Ust);
                  if (j != 1) Help();
                  break;
        case 'r': j=sscanf(pc+2, "%f", &Ra);
                  if (j != 1) Help();
                  break;
        case 'z': j=sscanf(pc+2, "%f", &Z0);
                  if (j != 1) Help();
                  break;
        case 'f': j=sscanf(pc+2, "%s", &Name);
                  if (j != 1) Help();
                  break;
        case 'v': j = 0;
                  pc += 2;
                  while (1 == sscanf(pc, "%f", &z)) {
                    while (*pc != ',') pc++;
                    pc++;
                    Zz[j++] = z;
                  }
                  Nz = j;
                  break;
      }
    }
    else
      Help();
  }
  profiles();
  printf("Program IBJ37838 finished.\n");
  exit(0);
}

#endif  /*###################################################################*/
