/* Lesen der Prairie-Gras-Daten */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

static int Ic[101];
static char Line[4000], *Names[100], *Header, *Data;
static char FileName[256], LasForm[256], Path[256];
static int N1=7, N2=10, Nv, Ll, Nl, Nn;
static float LasList[200][4];

static void Trim(        /* Blanks am Anfang und Ende beseitigen  */
unsigned char *t )       /* zu trimmende Zeichenkette             */
  {
  int l, ll, i;
  ll = strlen( (char*)t );
  for (l=ll-1; l>=0; l--)  if (t[l] != ' ')  break;
  t[l+1] = 0;
  ll = l+1;
  for (l=0; l<ll; l++)  if (t[l] > ' ') break;
  if (l > 0) {
    for (i=0; i<ll-l; i++)  t[i] = t[i+l];
    t[i] = 0; }
  return; }

int ReadPrairie( char *fn )
  {
  FILE *f;
  int l, i, n;
  f = fopen(fn, "rb");
  if (!f) {
    printf("File %s nicht lesbar!\n", fn);
    exit(1);
    }
  if (!fgets(Line, 4000, f))  return -1;
  Nn = 0;
  l = strlen(Line);
  Ll = l;
  for (i=0; i<l; i++)
    if (Line[i] == '.') {
      Nn++;
      if (Nn > 100)  return -2;
      Ic[Nn] = i;
      }
  for (i=1; i<N1; i++) {
    if (!fgets(Line, 4000, f))  return -3;
    l = strlen(Line);
    if (l > Ll)  Ll = l;
    }
  Header = malloc(l+1);  if (!Header)  return -4;
  strcpy(Header, Line);
  Ic[0] = -1;
  for (n=0; n<Nn; n++) {
    *Line = 0;
    Names[n] = Header+Ic[n]+1;
    strncat(Line, Names[n], Ic[n+1]-Ic[n]);
    Trim(Line);
    strcpy(Names[n], Line);
    }
  for (i=N1; i<N2-1; i++) {
    if (!fgets(Line, 4000, f))  return -5;
    l = strlen(Line);
    if (l > Ll)  Ll = l;
    }
  Nv = 0;
  while (fgets(Line, 4000, f)) {
    l = strlen(Line);
    if (l > Ll)  Ll = l;
    Trim(Line);
    if (*Line)  Nv++;
    }
  fseek(f, 0, SEEK_SET);
  Ll++;
  Data = malloc(Ll*Nv);  if (!Data)  return -6;
  for (i=0; i<N2-1; i++)
    if (!fgets(Line, 4000, f))  return -7;
  for (i=0; i<Nv; ) {
    if (!fgets(Data+i*Ll, Ll, f))  return -7;
    strcpy(Line, Data+i*Ll);
    Trim(Line);
    if (*Line)  i++;
    }
  fclose(f);
  return Nv;
  }

static int Value( int row, char *name, int seq, float *pv )
  {
  int i, n;
  char s[256];
  i = 0;
  *pv = 0;
  for (n=0; n<Nn; n++)
    if (!strcmp(Names[n], name)) {
      i++;
      if (i >= seq)  break;
      }
  if (n >= Nn)  return -1;
  *s = 0;
  strncat(s, Data+row*Ll+Ic[n]+1, Ic[n+1]-Ic[n]);
  Trim(s);
  if (!*s)  return 0;
  *pv = atof(s);
  return 1;
  }

int ReadList( char *fn )
  {
  FILE *f;
  int i, n;
  float x, cy, sy, cm;
  char buf[100];
  f = fopen(fn, "rb");
  if (!f) {
    fprintf(stderr, "can't read file %s!\n", fn);
    return -1;
    }
  i = 0;
  while(fgets(buf, 200, f)) if (buf[0] == '*') break;
  while(fgets(buf, 200, f)) {
    if (5 != sscanf(buf, "%d %f %f %f %f", &n, &x, &cy, &sy, &cm))  return -2;
    if (n != i+1)  return -3;
    LasList[i][0] = x;
    LasList[i][1] = cy;
    LasList[i][2] = sy;
    LasList[i][3] = cm;
    i++;
    if (i >= 200)  break;
    }
  fclose(f);
  return i;
  }

int CompareCon( char *fileform )
  {
  int i, k, iv, m, ix;
  float xp, us, lmo, hm, cy, syo, cmo, sya, cma, cyl, syl, cml;
  float u, x, w2p, a;
  char fn[256], ffn[3][256], *ps;
  static char *nn[3] = { "plume width Sigma-y", "integrated concentration Cy",
    "maximum concentration Cmax" };
  FILE *ff[3];
  static float xx[6] = { 0, 50, 100, 200, 400, 800 };
  strcpy(fn, fileform);
  ps = strrchr(fn, '\\');
  if (ps)  ps[1] = 0;
  else     *fn = 0;
  strcpy(ffn[0], fn);  strcat(ffn[0], "cmp-sy.txt");
  strcpy(ffn[1], fn);  strcat(ffn[1], "cmp-cy.txt");
  strcpy(ffn[2], fn);  strcat(ffn[2], "cmp-cm.txt");
  for (m=0; m<3; m++) {
    ff[m] = fopen(ffn[m], "w");
    if (!ff[m]) {
      printf("error creating file %s!\n", ffn[m]);
      return -4;
      }
    fprintf(ff[m], " N XP 1000/L      L   U*   U2");
    }
  fprintf(ff[0], " SYO050 SYA050 SYL050 SYO100 SYA100 SYL100 SYO200 SYA200 SYL200"
                 " SYO400 SYA400 SYL400 SYO800 SYA800 SYL800\n");
  fprintf(ff[1], " CYO050 CYL050 CYO100 CYL100 CYO200 CYL200"
                 " CYO400 CYL400 CYO800 CYL800\n");
  fprintf(ff[2], "   CMO050   CMA050   CML050   CMO100   CMA100   CML100"
                 "   CMO200   CMA200   CML200"
                 "   CMO400   CMA400   CML400   CMO800   CMA800   CML800\n");
  w2p = sqrt(2*3.1415926);
  printf(" N XP   HM  UST    LMO    U   X"
        "    CY   CYL SYO SYA SYL     CMO     CMA     CML\n");
  for (iv=0; iv<Nv; iv++) {
    i = Value(iv, "EXP", 0, &xp);
    if (i < 1) { printf("Zeile %d: YR nicht lesbar! (%d)\n", iv+1, i);  return -1; }
    i = Value(iv, "L", 0, &lmo);
    if (i < 1) { printf("Zeile %d: L nicht lesbar! (%d)\n", iv+1, i);  return -1; }
    i = Value(iv, "U", 0, &u);
    if (i < 1) { printf("Zeile %d: U nicht lesbar! (%d)\n", iv+1, i);  return -1; }
    i = Value(iv, "U*", 0, &us);
    if (i < 1) { printf("Zeile %d: U* nicht lesbar! (%d)\n", iv+1, i);  return -1; }
    i = Value(iv, "H", 0, &hm);
    if (i < 1) {
      if (lmo < 0)  hm = 800;
      else          hm = 0.3*sqrt(us*lmo*1.e4);
      }
    sprintf(fn, fileform, (int)xp);
    i = ReadList(fn);
    if (i < 0) {
      printf("error %d reading file %s\n", -i, fn);
      return -2;
      }
    Nl = i;
    for (m=0; m<3; m++)
      fprintf(ff[m], "%2d %2.0f %6.1f %6.1f %4.2f %4.1f", iv+1, xp, 1000/lmo, lmo, us, u);
    a = u*1.e-3;
    for (k=1; k<=5; k++) {
      x = xx[k];
      i = Value(iv, "C/Q",  k, &cmo);
      cmo = (i == 1) ? cmo*1.e9 : 0;
      i = Value(iv, "Cy/Q", k, &cy);
      cy = (i == 1) ? cy*1.e6 : 0;
      i = Value(iv, "SIGY", k, &syo);
      if (i < 1)  syo = 0;
      sya = (cmo > 0) ? 1.e3*cy/(w2p*cmo) : 0;
      if (syo < 0) {
        syo = sya;
        cma = cmo;
        }
      else  cma = (syo > 0) ? 1.e3*cy/(w2p*syo) : 0;
      for (i=0; i<Nl; i++)  if (LasList[i][0] == x)  break;
      if (i >= Nl) {
        printf("x=%1.0f not found in file %s!\n", x, fn);
        return -3;
        }
      cyl = LasList[i][1];
      syl = LasList[i][2];
      cml = LasList[i][3];
      printf("%2d %2.0f %4.0f %4.2f %6.1f %4.1f %3.0f",
        iv+1, xp, hm, us, lmo, u, x);
      printf(" %5.0f %5.0f %3.0f %3.0f %3.0f %7.0f %7.0f %7.0f\n",
        cy/10, cyl/10, syo, sya, syl, cmo/10, cma/10, cml/10);
      if (syo <= 0)  syo = sya;
      fprintf(ff[0], " %6.1f %6.1f %6.1f", syo, sya, syl);
      fprintf(ff[1], " %6.2f %6.2f", a*cy, a*cyl);
      fprintf(ff[2], " %8.2f %8.2f %8.2f", a*cmo, a*cma, a*cml);
      }
    for (m=0; m<3; m++)  fprintf(ff[m], "\n");
    }
  for (m=0; m<3; m++)  fclose(ff[m]);
  return 0;
  }

//--------------------------------------------------------- writeIBJparticle
int WriteIBJparticle( void )
  {
  int i, j, iv, n;
  float xp, us, lm, hm, u, us1, u1;
  FILE *f, *fdump;
  char fn[256], fndump[256], line[1024], *cp;
  n = 35;
  strcpy(fn, Path);
  strcat(fn, "prtcmd.txt");
  strcpy(fndump, Path);
  strcat(fndump, "profile.txt");
  f = fopen(fn, "w");
  if (!f) {
    printf("File %s nicht beschreibbar!\n", fn);
    return -1;
    }
  fprintf(f, "-- Eingabe-Datei fr IBJparticle\n");
  fprintf(f, "-- Validierung VDI 3783 Blatt 8: prairie\n");
  fprintf(f, "-- \n");
  fprintf(f, "*d  '-----\n");
  fprintf(f, "mx  185\n");
  fprintf(f, "my  201\n");
  fprintf(f, "mz  1\n");
  fprintf(f, "nz  %d\n", n);
  fprintf(f, "ti  Prairie-Grass\n");
  fprintf(f, "*g  '-----\n");
  fprintf(f, "a0  -32.5\n");
  fprintf(f, "b0  -502.5\n");
  fprintf(f, "cc  1.2 1.8\n");
  fprintf(f, "da  5\n");
  fprintf(f, "x0  -32.5\n");
  fprintf(f, "y0  -502.5\n");
  fprintf(f, "x1  892.5\n");
  fprintf(f, "y1  502.5\n");
  fprintf(f, "zz  0 0.1 0.2 0.4 0.6 0.8 1 1.2 1.8 2 2.2 2.5 3 3.5 4 5 6 7 8 9 10"
             "    12 15 20 30 40 50 70 100 200 300 400 500 600 700 800"
             "    900 1000 1200\n");
  fprintf(f, "*q '-----\n");
  fprintf(f, "hq  0.50\n");
  fprintf(f, "cq  0.10\n");
  fprintf(f, "xq  0\n");
  fprintf(f, "yq  0\n");
  fprintf(f, "*p '-----\n");
  fprintf(f, "rp 30000\n");
  fprintf(f, "ta 0.0\n");
  fprintf(f, "aa 0.3\n");

  for (iv=0; iv<Nv; iv++) {
    i = Value(iv, "EXP", 0, &xp);
    if (i < 1) printf("Zeile %d: EXP nicht lesbar!\n", iv);
    i = Value(iv, "U", 0, &u);
    if (i < 1) printf("Zeile %d: U nicht lesbar!\n", iv);
    i = Value(iv, "U*", 0, &us);
    if (i < 1) printf("Zeile %d: U* nicht lesbar!\n", iv);
    i = Value(iv, "L", 0, &lm);
    if (i < 1) printf("Zeile %d: L nicht lesbar!\n", iv);
    hm = 0.;
    i = Value(iv, "H", 0, &hm);
    if (xp == 15.) hm = 0.;
    fprintf(f, "*p '###### %2.0f\n", xp);
    sprintf(line, "..\\IBJ37838 -l%f -m%f -u%f -h%f -z0.008 -i600 -v%s%s%s -f%s",
      lm, hm, u, 2.0,
      "0,0.1,0.2,0.4,0.6,0.8,1,1.2,1.8,2,2.2,2.5,3,3.5,4,5,6,",
      "7,8,9,10,12,15,20,30,40,50,70,100,200,300,400,500,",
      "600,700,800,900,1000,1200",
      fndump);
    system(line);
    fdump = fopen(fndump, "r");
    i=0;
    while (fgets(line, 1024, fdump) != NULL) {
      if (i==0) {
        cp = strstr(line, "ust=");
        j=0;
        if (cp != NULL) j=sscanf(cp, "ust=%f", &u);
        printf("%2.0f %8.2f %8.2f %8.4f %8.4f\n", xp, lm, hm, u, us);
        if (j==1) us = u;
        fprintf(f, "vd %6.4f\n", us*0.02);
      }
      if (line[0] == 'z') continue;
      if (line[0] == 'r') continue;
      i++;
      fprintf(f, "%s", line);
    }
    fclose(fdump);
    remove(fndump);
    fprintf(f, "tu "); for (i=0; i<=n; i++) fprintf(f, "0.0 "); fprintf(f, "\n");
    fprintf(f, "tv "); for (i=0; i<=n; i++) fprintf(f, "0.0 "); fprintf(f, "\n");
    fprintf(f, "tw "); for (i=0; i<=n; i++) fprintf(f, "0.0 "); fprintf(f, "\n");
    fprintf(f, "** ############ %2.0f\n", xp);
    //fprintf(f, "*t *\n");
    fprintf(f, "*q\neq 5000\n*z\ndt 1\n*q\neq 0\n*z\ndt 4999\n");
    fprintf(f, "*w\nid \"%2.0f\"\nwc %2.0f\naw -2\nmo binary\nse i+,j+,k=1\n", xp, xp);
    fprintf(f, "*w cnc\n");
    fprintf(f, "*c all\n");
  }
  fprintf(f, "*e\n");
  fclose(f);
  return 0;
  }


int main( int argc, char *argv[] )
  {
  int i, n, do_compare, write_ibjparticle;
  char s[256], *pc;
  if (argc < 2) {
    printf("usage: anaprairie <PrairieFile> <options> ... \n");
    printf("       -C               create command file for IBJparticle\n");
    printf("                        (needs ..\IBJ37838.exe)\n");
    printf("       -v<InputFile>    compare results\n");
    exit(0);
    }
  write_ibjparticle = 0;
  do_compare = 0;
  for (i=1; i<argc; i++) {
    strcpy(s, argv[i]);
    if (*s == '-')
      switch (s[1]) {
        case 'C': write_ibjparticle = 1;
                  break;
        case 'v': do_compare = 1;
                  strcpy(LasForm, s+2);
                  break;
        default:  ;
        }
    else  strcpy(FileName, s);
    }
  n = ReadPrairie(FileName);
  if (n < 0) {
    printf("error %d\n", -n);
    exit(2);
    }
  strcpy(Path, FileName);
  for (i=strlen(Path)-1; i>=0; i--)
    if (Path[i] == '\\')  Path[i] = '/';
  pc = strrchr(Path, '/');
  if (pc)  pc[1] = 0;
  else     strcpy(Path, "./");
  if (write_ibjparticle)   WriteIBJparticle();
  else if (do_compare)  CompareCon(LasForm);
  return 0;
  }

