/* Analysieren der bodennahen Konzentration: Berechnung von Cy und Sy */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

static int Nx, Ny, TtlSz;
static float Xmin, Ymin, Delta;
static float *Data, *List;
static char *Header;
static char Path[256], InFile[256], OutFile[256];

#define  CC(i,j)  Data[j-1+Ny*(i-1)]
#define  LL(i,l)  List[l-1+4*(i-1)]

static int ReadData( char *name )
  {
  FILE *f;
  char fn[256], fn1[256];
  long l;
  strcpy(fn, Path);
  strcat(fn, name);
  strcat(fn, ".dmna");
  f = fopen(fn, "rb");  if (!f)         return -1;
  if (fseek(f, 0, SEEK_END))            return -10;
  l = ftell(f);
  Header = malloc(l+1);  if (!Header)   return -11;
  if (fseek(f, 0, SEEK_SET))            return -12;
  if (1 != fread(Header, l, 1, f))      return -13;
  fclose(f);
  Header[l] = 0;
  strcpy(fn, Path);
  strcat(fn, name);
  strcat(fn, ".dmnb");
  f = fopen(fn, "rb");  if (!f)         return -2;
  if (fseek(f, 0, SEEK_END))            return -20;
  TtlSz = ftell(f);
  Data = malloc(TtlSz);  if (!Data)     return -21;
  if (fseek(f, 0, SEEK_SET))            return -22;
  if (1 != fread(Data, TtlSz, 1, f))    return -23;
  fclose(f);
  return 0;
  }

static int StrIcmp( char *s1, char *s2, int n )
  {
  int i;
  if (!s1 || !s2)                          return 1;
  for (i=0; i<n; i++) {
    if (s1[i]==0 && s2[i]==0)              return 0;
    if (tolower(s1[i]) != tolower(s2[i]))  return 1;
    }
  return 0;
  }

static int ReadVar( char *buf, char *name, char *format, int nr, void *result )
  {
  int i, n, *pi, l;
  float *pf;
  double *pd;
  char *pc, *pe;
  l = strlen(name);  if (l<1 || l>79)           return -1;
  pc = buf;
  while (pc) {
    if (!StrIcmp(pc, name, l))  break;
    pc = strchr(pc, '\n');
    if (pc)  pc++;
    }
  if (!pc)                                      return 0;
  if      (!strcmp(format, "%d"))   pi = result;
  else if (!strcmp(format, "%f"))   pf = result;
  else if (!strcmp(format, "%lf"))  pd = result;
  else                                          return -2;
  pe = strchr(pc, '\n');
  if (pe)  l = pe-pc+1;
  else     l = strlen(pc);
  pe = malloc(l+1);  if (!pe)                   return -3;
  strncpy(pe, pc, l);
  pe[l] = 0;
  pc = strtok(pe, " ,\t\r\n");  if (!pc)        return -4;
  for (n=0; n<nr; n++) {
    pc = strtok(NULL, " ,\t\r\n");
    if (!pc)  break;
    if      (!strcmp(format, "%d")) {
      i = sscanf(pc, format, pi);
      if (i == 1)  pi++;
      }
    else if (!strcmp(format, "%f")) {
      i = sscanf(pc, format, pf);
      if (i == 1)  pf++;
      }
    else if (!strcmp(format, "%lf")) {
      i = sscanf(pc, format, pd);
      if (i == 1)  pd++;
      }
    else                                        return -2;
    if (i != 1)  break;
  }
  free(pe);
  return n;
  }

int CalcSigY( void )
  {
  float jm, da, dj, sumy, sumc, w2p;
  int i, j;
  List = malloc(Nx*4*sizeof(float));  if (!List)  return -1;
  jm = (Ny+1.0)/2;
  da = 1.0/12;
  w2p = sqrt(2*3.1415926);
  for (i=1; i<=Nx; i++) {
    sumc = 0;
    sumy = 0;
    for (j=1; j<=Ny; j++) {
      dj = j - jm;
      sumc += CC(i,j);
      sumy += CC(i,j)*(dj*dj + da);
      }
    LL(i,1) = Xmin + (i-0.5)*Delta;
    LL(i,2) = sumc*Delta;
    LL(i,3) = (sumc > 0) ? Delta*sqrt(sumy/sumc) : 0;
    LL(i,4) = (sumc > 0) ? LL(i,2)/(w2p*LL(i,3)) : 0;
    }
  return 0;
  }

int PrintList( char *name )
  {
  int i;
  FILE *f;
  char fn[256];
  if (*name) {
    strcpy(fn, Path);
    strcat(fn, name);
    f = fopen(fn, "w");
    if (!f) {
      printf("can't write to file %s!\n", fn);
      return -1;
      }
    }
  else f = stdout;
  fprintf(f, "%3s %10s %10s %10s %10s\n*\n",
    "Nr", "x", "Iy*1.e6", "Sy", "Cm*1.e9");
  for (i=1; i<=Nx; i++)
    fprintf(f, "%3d %10.0f %10.1f %10.1f %10.1f\n",
           i, LL(i,1), 1.e6*LL(i,2), LL(i,3), 1.e9*LL(i,4));
  if (f != stdout)  fclose(f);
  return 0;
  }

#define ERR(a) { errc=(a);  goto error; }

int main( int argc, char *argv[] )
  {
  char s[80];
  int i, l, n, errc, ii[2], jj[2], f1, f2;
  f1 = f2 = 0;
  if (argc < 3) {
    printf("usage: anasigy <Path> -i<InputFile> -o<OutputFile>\n");
    exit(0);
    }
  for (i=1; i<argc; i++) {
    strcpy(s, argv[i]);
    if (*s == '-')
      switch(s[1]) {
        case 'i':  strcpy(InFile, s+2);
                   break;
        case 'o':  strcpy(OutFile, s+2);
                   break;
        default:   ;
        }
    else {
      strcpy(Path, s);
      l = strlen(Path);
      if (Path[l-1]!= '/' && Path[l-1]!='\\')  strcat(Path, "\\");
      }
    }
  i = ReadData(InFile);
  if (i < 0) {
    printf("error reading file %s%s! (%d)\n", Path, InFile, i);
    exit(1);
    }
  if (2 != ReadVar(Header, "lowb", "%d", 2, ii))      ERR(1);
  if (2 != ReadVar(Header, "hghb", "%d", 2, jj))      ERR(2);
  Nx = jj[0]-ii[0]+1;
  Ny = jj[1]-ii[1]+1;
  if (Nx*Ny*4 != TtlSz)                                ERR(13);
  if (1 != ReadVar(Header, "xmin",  "%f", 1, &Xmin) &&
      1 != ReadVar(Header, "xa",  "%f", 1, &Xmin))     ERR(3);
  if (1 != ReadVar(Header, "ymin",  "%f", 1, &Ymin)&&
      1 != ReadVar(Header, "ya",  "%f", 1, &Ymin))     ERR(4);
  if (1 != ReadVar(Header, "delta", "%f", 1, &Delta) &&
      1 != ReadVar(Header, "dd",  "%f", 1, &Delta))    ERR(5);
  printf("nx=%d, ny=%d, xmin=%f, ymin=%f, delta=%f\n",
  Nx, Ny, Xmin, Ymin, Delta);
  i = CalcSigY();  if (i < 0)  ERR(6);
  PrintList(OutFile);

  fprintf(stderr, "finished\n");
  return 0;
error:
  printf("error %d\n", errc);
  return errc;
  }
